﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Application
{
    using Application.DataSource.DTO;
    using Application.UI.DTO;
    using DbAccess.ReviewDbAccess;
    using DomainModel.ReviewDomainModel.Entity;
    using DomainModel.ReviewDomainModel.EntityFactory;

    public class ReviewApplication : ApplicationBase //建议不要用静态类，为了以后的扩展
    {
        public List<ReviewUiDto> GetUiReviews()
        {
            ReviewCollection reviewCollec = this.QueryDbReviewDtoAndTransformDoaminModel();
            List<Review> reviews = reviewCollec.GetUiReviews();//在DomainModel中执行所有的业务逻辑操作

            return this.TransformUiDto(reviews);//返回UI DTO
        }

        public List<ReviewUiDto> GetServiceReviews()
        {
            ReviewCollection reviewCollec = this.QueryDbReviewDtoAndTransformDoaminModel();
            List<Review> reviews = reviewCollec.GetServiceReviews();//在DomainModel中执行所有的业务逻辑操作

            return this.TransformUiDto(reviews);//返回UI DTO
        }

        public List<ReviewUiDto> GetUiAndServiceReviews()
        {
            ReviewCollection reviewCollec = this.QueryDbReviewDtoAndTransformDoaminModel();
            List<Review> reviews = reviewCollec.GetUiAndServiceReviews();//在DomainModel中执行所有的业务逻辑操作

            return this.TransformUiDto(reviews);//返回UI DTO
        }

        private ReviewCollection QueryDbReviewDtoAndTransformDoaminModel()
        {
            ReviewDbAccess access = new ReviewDbAccess();
            List<ReviewDbDto> dto = access.QueryReviewDto();//获取数据实体

            return this.TransformDomainModel(dto); //转换成DomainModel
        }

        protected virtual ReviewCollection TransformDomainModel(List<ReviewDbDto> dtoList)//可封装、重写  功能
        {
            ReviewCollection collection = new ReviewCollection();
            ReviewFactory factory = new ReviewFactory();
            dtoList.ForEach(dto =>
            {
                Review review = factory.TransformReview(dto.LastEditDate, dto.Title, dto.Id);
                if (review != null)
                    collection.Add(review);
            });
            return collection;
        }

        protected virtual List<ReviewUiDto> TransformUiDto(List<Review> reviewList)//可封装、重写 功能
        {
            List<ReviewUiDto> result = new List<ReviewUiDto>();
            reviewList.ForEach(review =>
           {
               result.Add(new ReviewUiDto()
               {
                   LastEditDate = review.LastEditDate.ToString(),
                   Title = review.Title.ShowTitle
               });
           });
            return result;
        }
    }
}